package com.dzeio.openhealth.ui.extension

import android.app.ProgressDialog
import android.content.Context
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContract
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.navArgs
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentExtensionBinding
import com.dzeio.openhealth.extensions.Extension
import com.dzeio.openhealth.extensions.ExtensionFactory
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@AndroidEntryPoint
class ExtensionFragment :
    BaseFragment<ExtensionViewModel, FragmentExtensionBinding>(ExtensionViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentExtensionBinding =
        FragmentExtensionBinding::inflate

    private val args: ExtensionFragmentArgs by navArgs()

    private val extension by lazy {
        ExtensionFactory.getExtension(args.extension)
            ?: throw Exception("No Extension found!")
    }

    private var request: ActivityResultLauncher<Any>? = null

    override fun onAttach(context: Context) {
        if (this.extension.contract != null) {
            this.request =
                registerForActivityResult<Any, Any>(this.extension.contract!! as ActivityResultContract<Any, Any>) {
                this.extension.onActivityResult(it as Any)
            }
        }
        super.onAttach(context)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val enabled = extension.enable(this)

        if (!enabled) {
            throw Exception("Extension can't be enabled (${extension.id})")
        }

        requireActivity().actionBar?.title = extension.name

//        extension.init(requireActivity())

        binding.importButton.setOnClickListener {
            val dialog = ProgressDialog(requireContext())
            dialog.setTitle("Importing...")
            dialog.setMessage("Imported 0 values")
            dialog.show()
            lifecycleScope.launch {
                extension.importWeight().collectLatest { state ->
                    Log.d("ExtensionFragment", state.state.name)
                    dialog.setMessage(state.statusMessage ?: "progress ${state.progress}%")
                    if (state.state == Extension.TaskState.DONE) {
                        dialog.setMessage("Finishing Import...")
                        lifecycleScope.launchWhenStarted {
                            state.additionalData!!.forEach {
                                it.source = extension.id
                                viewModel.importWeight(it)
                            }
                            dialog.dismiss()
                        }
                    }
                }
            }
        }

        lifecycleScope.launch {
            if (!extension.permissionsGranted() && request != null) {
                request!!.launch(extension.requestInput)
            }
        }
    }
}
