package com.dzeio.openhealth.ui.extensions

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.adapters.ExtensionAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentExtensionsBinding
import com.dzeio.openhealth.extensions.Extension
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.launch

@AndroidEntryPoint
class ExtensionsFragment :
    BaseFragment<ExtensionsViewModel, FragmentExtensionsBinding>(ExtensionsViewModel::class.java) {

    companion object {
        const val TAG = "ExtensionsFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentExtensionsBinding =
        FragmentExtensionsBinding::inflate

    private lateinit var activeExtension: Extension

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = ExtensionAdapter(viewModel.config)
        adapter.onItemClick = {
            activeExtension = it
            activeExtension.enable(this)
            Log.d(TAG, "${it.id}: ${it.name}")

            lifecycleScope.launch {
                extensionIsConnected(it)
            }
        }
        recycler.adapter = adapter

        val list = viewModel.extensions
        list.forEach {
            it.enable(this)
        }

        adapter.set(list)
    }

    private suspend fun extensionIsConnected(it: Extension) {
        // check if it is connected
        if (it.isConnected()) {
            gotoExtension(it)
            return
        }

        val ld = it.connect()
        if (ld) {
            gotoExtension(it)
        }
        // handle if extension can't be connected
    }

    private fun gotoExtension(it: Extension) {
        findNavController().navigate(
            ExtensionsFragmentDirections.actionNavExtensionsToNavExtension(it.id)
        )
    }
}
